import { createHttpLink, ApolloLink, ApolloClient, from, InMemoryCache } from '@apollo/client';
import { setContext } from '@apollo/client/link/context';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
}

const authLink = setContext((_, { headers }) => {
    // noop implementation
    return {
        headers,
    };
});

const httpLink = createHttpLink({
    uri: "/graphql",
    credentials: "same-origin",
});

function createLocaleLink(i18nProvider) {
    return new ApolloLink((operation, forward) => {
        operation.setContext(({ headers = {} }) => ({
            headers: Object.assign(Object.assign({}, headers), { "accept-language": i18nProvider.getLocale() }),
        }));
        return forward(operation);
    });
}

function createAmplicodeApolloClient(options) {
    const { cache, defaultOptions, i18nProvider } = options, other = __rest(options, ["cache", "defaultOptions", "i18nProvider"]);
    const links = i18nProvider != null
        ? [authLink, createLocaleLink(i18nProvider), httpLink]
        : [authLink, httpLink];
    return new ApolloClient(Object.assign({ link: from(links), cache: cache !== null && cache !== void 0 ? cache : new InMemoryCache(), defaultOptions: defaultOptions !== null && defaultOptions !== void 0 ? defaultOptions : {
            query: {
                fetchPolicy: "network-only",
            },
            watchQuery: {
                fetchPolicy: "network-only",
            },
        } }, other));
}

export { authLink, createAmplicodeApolloClient, createLocaleLink, httpLink };
